@@ -41,10 +41,12 @@ struct ieee802_11_elems {
 	u8 wpa_ie_len;
 	u8 *rsn_ie;
 	u8 rsn_ie_len;
-	u8 *wme;
-	u8 wme_len;
-	u8 *wme_tspec;
-	u8 wme_tspec_len;
+	u8 *wmm; /* WMM Information or Parameter Element */
+	u8 wmm_len; /* 7 = WMM Information; 24 = WMM Parameter */
+	u8 *wmm_tspec;
+	u8 wmm_tspec_len;
+	u8 *wps_ie;
+	u8 wps_ie_len;
 	u8 *power_cap;
 	u8 power_cap_len;
 	u8 *supp_channels;
@@ -53,12 +55,14 @@ struct ieee802_11_elems {
 	u8 mdie_len;
 	u8 *ftie;
 	u8 ftie_len;
+	u8 *timeout_int;
+	u8 timeout_int_len;
 	u8 *ht_capabilities;
 	u8 ht_capabilities_len;
 	u8 *ht_operation;
 	u8 ht_operation_len;
-	u8 *assoc_comeback;
-	u8 assoc_comeback_len;
+	u8 *vendor_ht_cap;
+	u8 vendor_ht_cap_len;
 };
 
 typedef enum { ParseOK = 0, ParseUnknown = 1, ParseFailed = -1 } ParseRes;
