@@ -459,9 +459,9 @@ static void ieee80211_send_assoc(struct wpa_supplicant *wpa_s)
 		*pos++ = 0x00; /* Microsoft OUI 00:50:F2 */
 		*pos++ = 0x50;
 		*pos++ = 0xf2;
-		*pos++ = 2; /* WME */
-		*pos++ = 0; /* WME info */
-		*pos++ = 1; /* WME ver */
+		*pos++ = 2; /* WMM */
+		*pos++ = 0; /* WMM info */
+		*pos++ = 1; /* WMM ver */
 		*pos++ = 0;
 	}
 
@@ -1087,9 +1087,10 @@ static void ieee80211_rx_mgmt_assoc_resp(struct wpa_supplicant *wpa_s,
 			   status_code);
 #ifdef CONFIG_IEEE80211W
 		if (status_code == WLAN_STATUS_ASSOC_REJECTED_TEMPORARILY &&
-		    elems.assoc_comeback && elems.assoc_comeback_len == 4) {
+		    elems.timeout_int && elems.timeout_int_len == 5 &&
+		    elems.timeout_int[0] == WLAN_TIMEOUT_ASSOC_COMEBACK) {
 			u32 tu, ms;
-			tu = WPA_GET_LE32(elems.assoc_comeback);
+			tu = WPA_GET_LE32(elems.timeout_int + 1);
 			ms = tu * 1024 / 1000;
 			wpa_printf(MSG_DEBUG, "MLME: AP rejected association "
 				   "temporarily; comeback duration %u TU "
@@ -1174,9 +1175,9 @@ static void ieee80211_rx_mgmt_assoc_resp(struct wpa_supplicant *wpa_s,
 #if 0 /* FIX? */
 	sta->assoc_ap = 1;
 
-	if (elems.wme && wpa_s->mlme.wmm_enabled) {
-		sta->flags |= WLAN_STA_WME;
-		ieee80211_sta_wmm_params(wpa_s, elems.wme, elems.wme_len);
+	if (elems.wmm && wpa_s->mlme.wmm_enabled) {
+		sta->flags |= WLAN_STA_WMM;
+		ieee80211_sta_wmm_params(wpa_s, elems.wmm, elems.wmm_len);
 	}
 #endif
 
@@ -1487,18 +1488,18 @@ static void ieee80211_bss_info(struct wpa_supplicant *wpa_s,
 		bss->rsn_ie_len = 0;
 	}
 
-	if (elems.wme &&
-	    (bss->wmm_ie == NULL || bss->wmm_ie_len != elems.wme_len ||
-	     os_memcmp(bss->wmm_ie, elems.wme, elems.wme_len))) {
+	if (elems.wmm &&
+	    (bss->wmm_ie == NULL || bss->wmm_ie_len != elems.wmm_len ||
+	     os_memcmp(bss->wmm_ie, elems.wmm, elems.wmm_len))) {
 		os_free(bss->wmm_ie);
-		bss->wmm_ie = os_malloc(elems.wme_len + 2);
+		bss->wmm_ie = os_malloc(elems.wmm_len + 2);
 		if (bss->wmm_ie) {
-			os_memcpy(bss->wmm_ie, elems.wme - 2,
-				  elems.wme_len + 2);
-			bss->wmm_ie_len = elems.wme_len + 2;
+			os_memcpy(bss->wmm_ie, elems.wmm - 2,
+				  elems.wmm_len + 2);
+			bss->wmm_ie_len = elems.wmm_len + 2;
 		} else
 			bss->wmm_ie_len = 0;
-	} else if (!elems.wme && bss->wmm_ie) {
+	} else if (!elems.wmm && bss->wmm_ie) {
 		os_free(bss->wmm_ie);
 		bss->wmm_ie = NULL;
 		bss->wmm_ie_len = 0;
@@ -1594,9 +1595,9 @@ static void ieee80211_rx_mgmt_beacon(struct wpa_supplicant *wpa_s,
 		wpa_s->mlme.cts_protect_erp_frames = use_protection;
 	}
 
-	if (elems.wme && wpa_s->mlme.wmm_enabled) {
-		ieee80211_sta_wmm_params(wpa_s, elems.wme,
-					 elems.wme_len);
+	if (elems.wmm && wpa_s->mlme.wmm_enabled) {
+		ieee80211_sta_wmm_params(wpa_s, elems.wmm,
+					 elems.wmm_len);
 	}
 }
 
@@ -1708,7 +1709,6 @@ static void ieee80211_rx_mgmt_ft_action(struct wpa_supplicant *wpa_s,
 		wpa_printf(MSG_DEBUG, "MLME: Foreign STA Address " MACSTR
 			   " in FT Action Response", MAC2STR(sta_addr));
 		return;
-			   
 	}
 
 	if (status) {
@@ -1738,9 +1738,9 @@ static void ieee80211_rx_mgmt_ft_action(struct wpa_supplicant *wpa_s,
 
 #ifdef CONFIG_IEEE80211W
 
-/* MLME-PING.response */
-static int ieee80211_sta_send_ping_resp(struct wpa_supplicant *wpa_s,
-					const u8 *addr, const u8 *trans_id)
+/* MLME-SAQuery.response */
+static int ieee80211_sta_send_sa_query_resp(struct wpa_supplicant *wpa_s,
+					    const u8 *addr, const u8 *trans_id)
 {
 	struct ieee80211_mgmt *mgmt;
 	int res;
@@ -1749,7 +1749,7 @@ static int ieee80211_sta_send_ping_resp(struct wpa_supplicant *wpa_s,
 	mgmt = os_zalloc(sizeof(*mgmt));
 	if (mgmt == NULL) {
 		wpa_printf(MSG_DEBUG, "MLME: Failed to allocate buffer for "
-			   "ping action frame");
+			   "SA Query action frame");
 		return -1;
 	}
 
@@ -1759,11 +1759,11 @@ static int ieee80211_sta_send_ping_resp(struct wpa_supplicant *wpa_s,
 	os_memcpy(mgmt->bssid, wpa_s->bssid, ETH_ALEN);
 	mgmt->frame_control = IEEE80211_FC(WLAN_FC_TYPE_MGMT,
 					   WLAN_FC_STYPE_ACTION);
-	mgmt->u.action.category = WLAN_ACTION_PING;
-	mgmt->u.action.u.ping_resp.action = WLAN_PING_RESPONSE;
-	os_memcpy(mgmt->u.action.u.ping_resp.trans_id, trans_id,
-		  WLAN_PING_TRANS_ID_LEN);
-	len += 1 + sizeof(mgmt->u.action.u.ping_resp);
+	mgmt->u.action.category = WLAN_ACTION_SA_QUERY;
+	mgmt->u.action.u.sa_query_resp.action = WLAN_SA_QUERY_RESPONSE;
+	os_memcpy(mgmt->u.action.u.sa_query_resp.trans_id, trans_id,
+		  WLAN_SA_QUERY_TR_ID_LEN);
+	len += 1 + sizeof(mgmt->u.action.u.sa_query_resp);
 
 	res = ieee80211_sta_tx(wpa_s, (u8 *) mgmt, len);
 	os_free(mgmt);
@@ -1772,36 +1772,36 @@ static int ieee80211_sta_send_ping_resp(struct wpa_supplicant *wpa_s,
 }
 
 
-static void ieee80211_rx_mgmt_ping_action(
+static void ieee80211_rx_mgmt_sa_query_action(
 	struct wpa_supplicant *wpa_s, struct ieee80211_mgmt *mgmt, size_t len,
 	struct ieee80211_rx_status *rx_status)
 {
-	if (len < 24 + 1 + sizeof(mgmt->u.action.u.ping_req)) {
-		wpa_printf(MSG_DEBUG, "MLME: Too short Ping Action frame");
+	if (len < 24 + 1 + sizeof(mgmt->u.action.u.sa_query_req)) {
+		wpa_printf(MSG_DEBUG, "MLME: Too short SA Query Action frame");
 		return;
 	}
 
-	if (mgmt->u.action.u.ping_req.action != WLAN_PING_REQUEST) {
-		wpa_printf(MSG_DEBUG, "MLME: Unexpected Ping Action %d",
-			   mgmt->u.action.u.ping_req.action);
+	if (mgmt->u.action.u.sa_query_req.action != WLAN_SA_QUERY_REQUEST) {
+		wpa_printf(MSG_DEBUG, "MLME: Unexpected SA Query Action %d",
+			   mgmt->u.action.u.sa_query_req.action);
 		return;
 	}
 
 	if (os_memcmp(mgmt->sa, wpa_s->bssid, ETH_ALEN) != 0) {
-		wpa_printf(MSG_DEBUG, "MLME: Ignore ping from unknown source "
-			   MACSTR, MAC2STR(mgmt->sa));
+		wpa_printf(MSG_DEBUG, "MLME: Ignore SA Query from unknown "
+			   "source " MACSTR, MAC2STR(mgmt->sa));
 		return;
 	}
 
 	if (wpa_s->mlme.state == IEEE80211_ASSOCIATE) {
-		wpa_printf(MSG_DEBUG, "MLME: Ignore ping request during "
+		wpa_printf(MSG_DEBUG, "MLME: Ignore SA query request during "
 			   "association process");
 		return;
 	}
 
-	wpa_printf(MSG_DEBUG, "MLME: Replying to ping request");
-	ieee80211_sta_send_ping_resp(wpa_s, mgmt->sa,
-				     mgmt->u.action.u.ping_req.trans_id);
+	wpa_printf(MSG_DEBUG, "MLME: Replying to SA Query request");
+	ieee80211_sta_send_sa_query_resp(wpa_s, mgmt->sa, mgmt->u.action.u.
+					 sa_query_req.trans_id);
 }
 
 #endif /* CONFIG_IEEE80211W */
@@ -1824,8 +1824,8 @@ static void ieee80211_rx_mgmt_action(struct wpa_supplicant *wpa_s,
 		break;
 #endif /* CONFIG_IEEE80211R */
 #ifdef CONFIG_IEEE80211W
-	case WLAN_ACTION_PING:
-		ieee80211_rx_mgmt_ping_action(wpa_s, mgmt, len, rx_status);
+	case WLAN_ACTION_SA_QUERY:
+		ieee80211_rx_mgmt_sa_query_action(wpa_s, mgmt, len, rx_status);
 		break;
 #endif /* CONFIG_IEEE80211W */
 	default:
